import gtk
import gobject

class QueryController(gobject.GObject):

    _host = ""
    _user = ""
    _passw = ""
    _root = ""
    _query = []
    _labels = []

    @property
    def root(self):
        return self._root

    @root.setter
    def root(self, new_root):
        self._root = new_root

    @property
    def labels(self):
        return self._labels

    @labels.setter
    def labels(self, new_labels):
        self._labels = new_labels

    @property
    def query(self):
        return self._query

    @query.setter
    def query(self, new_query):
        self._query = new_query

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, new_host):
        self._host = new_host

    @property
    def credentials_tuple(self):
        return (self._user, self._passw)

    @credentials_tuple.setter
    def credentials_tuple(self, creds):
        self._user, self._passw = creds

    @property
    def credentials(self):
        return "%s:%s" % self.credentials_tuple

    @credentials.setter
    def credentials(self, new_creds):
        self._user, _, self._passw = new_creds.partition(":")

    def send_query(self, *_):

        def on_search_query(host, credentials, root_type, query, fields):
            print (host, credentials, root_type, query, fields)
            user, passw = credentials

            import xsalogic.queries as queries
            import xsa.errors as xsa_errors

            try:
                query_results = queries.search_for( host,
                                                    root_type,
                                                    query,
                                                    fields,
                                                    user,
                                                    passw
                                                    )
            except xsa_errors.Error as e:
                md = gtk.MessageDialog( type=gtk.MESSAGE_WARNING,
                                        buttons=gtk.BUTTONS_OK,
                                        message_format=str(e)
                                        )
                md.run()
                md.destroy()
                self.results = []
            except Exception:
                md = gtk.MessageDialog( type=gtk.MESSAGE_WARNING,
                                        buttons=gtk.BUTTONS_OK,
                                        message_format="An unexpected Error occured."
                                        )
                md.run()
                md.destroy()
                self.results = []
            else:
                self.results = query_results

            self.resultsview.show_data(self.results, root_type)
            self.statusbar.push(0, "%s Results"%len(self.results))
            self.chartview.update_chart(self.results, root_type)

        self.root=self.queryview.combobox.get_selected_item()
        self.query=self.queryview.treeView_search.get_query(self.queryview.combobox.get_selected_item())
        self.labels=self.queryview.treeView_fields.get_selected_fields()

        import xsalogic.datatypereader as type_reader
        fields = type_reader.get_fields_from_labels(self.labels,self.root)
        on_search_query(self.host,                             self.credentials_tuple,
                                self.root,
                                self.query,
                                fields)


    def __init__(self, main, queryview, menuview, chartview, resultsview, statusbar):
        super(QueryController, self).__init__()
        self.main = main
        self.queryview = queryview
        self.menuview = menuview
        self.chartview = chartview
        self.resultsview = resultsview
        self.statusbar = statusbar


        def callback_edit_Server_Settings(*_):
            dialog = gtk.Dialog(    "Server Settings",
                                    None, 0,
                                    (   gtk.STOCK_OK, gtk.RESPONSE_OK,
                                        "Cancel", gtk.RESPONSE_CANCEL)
                                    )
            dialog.set_resizable(False)
            dialog.set_size_request(330, 180)
            hbox = gtk.HBox(False, 8)
            hbox.set_border_width(8)
            dialog.vbox.pack_start(hbox, False, False, 0)

            stock = gtk.image_new_from_stock(
                    gtk.STOCK_DIALOG_AUTHENTICATION,
                    gtk.ICON_SIZE_DIALOG)
            hbox.pack_start(stock, False, False, 0)

            table = gtk.Table(2, 3)
            table.set_row_spacings(4)
            table.set_col_spacings(4)
            hbox.pack_start(table, True, True, 0)

            label_user =  gtk.Label()
            label_user.set_text("User:")
            table.attach(label_user, 0, 1, 0, 1)

            entry_user = gtk.Entry()
            entry_user.set_text(self.credentials_tuple[0])
            table.attach(entry_user, 1, 2, 0, 1)

            label_passw =  gtk.Label()
            label_passw.set_text("Password:")
            table.attach(label_passw, 0, 1, 1, 2)

            entry_passw = gtk.Entry()
            entry_passw.set_visibility(False)
            entry_passw.set_text(self.credentials_tuple[1])
            table.attach(entry_passw, 1, 2, 1, 2)

            label_host =  gtk.Label()
            label_host.set_text("Host:")
            table.attach(label_host, 0, 1, 2, 3)

            entry_host = gtk.Entry()
            entry_host.set_text(self.host)
            table.attach(entry_host, 1, 2, 2, 3)

            dialog.show_all()
            response = dialog.run()

            if response == gtk.RESPONSE_OK:
                self.credentials_tuple = (entry_user.get_text(),entry_passw.get_text())
                self.host = entry_host.get_text()

            dialog.destroy()

        def event_clicked_toggle_selection(*_):
            self.resultsview.toggle_selection()

        def event_clicked_download(*_):
            import gobject
            import xsalogic.queries as queries
            import xsalogic.datatypereader as type_reader

            dialog = gtk.FileChooserDialog( "Open..",
                                            None,
                                            gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                            gtk.STOCK_OPEN, gtk.RESPONSE_OK))
            dialog.set_default_response(gtk.RESPONSE_OK)

            response = dialog.run()
            if response == gtk.RESPONSE_OK:
                rest = type_reader.get_rest(self.root)
                enabled = (d for d in self.resultsview.get_store() if d[0])
                for d in enabled:
                    def stop_spinner(_row):
                        _row[2] = False # Hide spinner
                        _row[0] = False # Checkbox to false
                    d[3] = 0
                    d[2] = True # Show Spinner
                    queries.download_async( d[1],
                                            self.host,
                                            self.credentials_tuple,
                                            rest,
                                            dialog.get_filename(),
                                            cb=stop_spinner,
                                            cb_args=(d,)
                                            )
            dialog.destroy()

        self.resultsview.toggle_selection()

        self.menuview.connect("spawn-connection-dialog", callback_edit_Server_Settings)
        self.menuview.connect("send-query", self.send_query)
        self.menuview.connect("toggle-selection", event_clicked_toggle_selection)
        self.menuview.connect("download-selection", event_clicked_download)
